// Copyright (C) 2013 by FZI Forschungszentrum Informatik am Karlsruher Institut fuer Technologie
// 
// Author: Benjamin Ranft (benjamin.ranft@web.de)
// 
// This file is part of demoHeterogeneousParallelism.
// 
// demoHeterogeneousParallelism is free software: you can redistribute it and/or modify it under the terms of the GNU 
// General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your 
// option) any later version.
// 
// demoHeterogeneousParallelism is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
// even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
// for more details.
// 
// You should have received a copy of the GNU General Public License along with demoHeterogeneousParallelism. If not, 
// see <http://www.gnu.org/licenses/>.


// place-holder for your code: feature matching sample application
// ===============================================================

#pragma once

#include "hawaii/GPU/openCV.h"
#include <opencv2/features2d/features2d.hpp>

// parameters for feature detection and extraction
struct DetectExtractOptions {
	
	// see "http://docs.opencv.org/modules/features2d/doc/feature_detection_and_description.html#orb-orb" for details
	int   nFeatures     ;
	float scaleFactor   ;
	int   nLevels       ;
	int   edgeThreshold ;
	int   firstLevel    ;
	int   WTA_K         ;
	int   scoreType     ;
	int   patchSize     ;
	
	// c'tor with values suitable for optical flow (OpenCV's commented defaults are more useful for object recognition.) 
	DetectExtractOptions() :
		nFeatures(     512                   ),  // 500
		scaleFactor(   1.2f                  ),  // 1.2f
		nLevels(       3                     ),  // 8
		edgeThreshold( 15                    ),  // 31
		firstLevel(    0                     ),  // 0
		WTA_K(         2                     ),  // 2
		scoreType(     cv::ORB::HARRIS_SCORE ),  // cv::ORB::HARRIS_SCORE
		patchSize(     31                    ) { // 31
	}
} ;

// detect and extract keypoints from one image
// developer note: Wrapped OpenCV code does not support streams for asynchronous operation.
void detectExtractCPU( const cv::Mat                      image,
	                          std::vector< cv::KeyPoint >& keypoints,
	                          cv::Mat&                     descriptors,
	                    const DetectExtractOptions         options     = DetectExtractOptions() ) ;
void detectExtractGPU( const cv::gpu::GpuMat              image,
	                          std::vector< cv::KeyPoint >& keypoints,
	                          cv::gpu::GpuMat&             descriptors,
	                    const DetectExtractOptions         options     = DetectExtractOptions() ) ;

// find correspondences between features
void matchCPU( const cv::Mat          descriptorsCurr,
               const cv::Mat          descriptorsPrev,
                     cv::Mat&         indices,
                     cv::Mat&         distances       ) ;
void matchGPU( const cv::gpu::GpuMat& descriptorsCurr,
               const cv::gpu::GpuMat& descriptorsPrev,
                     cv::gpu::GpuMat& indices,
                     cv::gpu::GpuMat& distances,
                     cv::gpu::Stream& stream          = cv::gpu::Stream::Null() ) ;

// visualize found matches
void drawMatches( const cv::Mat                      imageCurr,
                  const std::vector< cv::KeyPoint >& keypointsCurr,
                  const std::vector< cv::KeyPoint >& keypointsPrev,
                  const cv::Mat                      indices,
                        cv::Mat&                     visualization ) ;
