// Copyright (C) 2013 by FZI Forschungszentrum Informatik am Karlsruher Institut fuer Technologie
// 
// Author: Benjamin Ranft (benjamin.ranft@web.de)
// 
// This file is part of libHawaii.
// 
// libHawaii is free software: you can redistribute it and/or modify it under the terms of the GNU General Public 
// License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later 
// version.
// 
// libHawaii is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with libHawaii. If not, see 
// <http://www.gnu.org/licenses/>.


// information on this system's CPUs and GPUs
// ==========================================

#pragma once

#include <cstddef>

namespace hawaii {

// number of physical CPUs (usually 1 for notebooks and desktops)
extern const int CPUSockets ;

// number of real cores
// user note: use "CPUCores" (across all sockets) for maximum performance or "CPUCoresPerSocket" for best 
//            scaling/efficiency, especially if cache locality is important
extern const int CPUCores          ;
extern const int CPUCoresPerSocket ;

// number of register sets
// user note: use these instead of "CPUCores*" if your task often needs to wait for IO etc.
extern const int CPUThreads          ;
extern const int CPUThreadsPerSocket ;

// thermal design power of all CPUs combined
// user note: If your CPU is yet unknown, you will be prompted for a single CPU's TDP though.
extern const double CPUPower ;

// number of GPUs present in the system
// user note: 0 if compile-time GPU support is disabled, limited by HAWAII_GPUSMAX below
extern const int GPUs ;

// maximum number of GPUs supported by "libHawaii"
#define HAWAII_GPUS_MAX 64

// determine if GPU has own instead of sharing CPU memory
// user note: A negative index means the currently active GPU.
bool GPUMemoryOwn( int GPU = -1 ) ;

// GPU's compute capability: same format as NVCC's "__CUDA_ARCH__" macro, i.e. CC 2.1 yields 210
// user note: see above
int GPUComputeCapability( int GPU = -1 ) ;

// minimum required alignment for textures or surfaces, either for a specific GPU or the maximum among all GPUs
// user note: see above
size_t GPUAlignment( int GPU = -1 ) ;
size_t GPUAlignmentMax() ;

// GPU's support for unified addressing
// user note: see above
bool GPUUnifiedAddressing( int GPU = -1 ) ;

// thermal design power of each GPU
// user note: see above
double GPUPower( int GPU = -1 ) ;

} // namespace "hawaii"
