// Copyright (C) 2013 by FZI Forschungszentrum Informatik am Karlsruher Institut fuer Technologie
// 
// Author: Benjamin Ranft (benjamin.ranft@web.de)
// 
// This file is part of libHawaii.
// 
// libHawaii is free software: you can redistribute it and/or modify it under the terms of the GNU General Public 
// License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later 
// version.
// 
// libHawaii is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied 
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with libHawaii. If not, see 
// <http://www.gnu.org/licenses/>.


// series of processing steps to be executed sequentially
// ======================================================

#include "hawaii/flow/series.h"

namespace hawaii {
namespace flow {

// "Series": combine several processing steps into one
// ---------------------------------------------------

// reset internal state
void Series::reset() {
	for( auto& step : this->steps ) { step->reset() ; }
	StepBase::reset() ;
}

// subsequently call all steps
void Series::process( const ItemBase::Ptr& srcPtr,
                            ItemBase::Ptr& dstPtr ) {
	// check if initialized
	HAWAII_ERROR_CONDITIONAL( this->steps.empty(), "no processing steps to call" )
	
	// intermediate work items
	ItemBase::Ptr intSrcPtr, intDstPtr ;
	
	// go through all processing steps
	intSrcPtr = srcPtr ;
	for( auto step : this->steps ) {
		
		// actual processing, except for final item
		if( HAWAII_LIKELY( !intSrcPtr->isFinal() ) ) {
			step->activateGPUTry() ;
			step->process( intSrcPtr, intDstPtr ) ;
		} else {
			intDstPtr = itemFinalPtr ;
			break ;
		}
		intSrcPtr = intDstPtr ;
	}
	dstPtr = intDstPtr ;
	
} // method "Series::process()"

} } // namespaces "hawaii::flow"
